/*
 * Filter
 * Copyright (C) 2011 by Apaq
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package dk.apaq.filter.jpa.interpreters;

import dk.apaq.filter.core.LikeFilter;
import dk.apaq.filter.jpa.WhereClause;

/**
 *
 * @author michael
 */
public class LikeFilterInterpreter extends AbstractInterpreter<LikeFilter> {

    private static final char ESCAPECHAR = '!';

    public boolean interpret(WhereClause clause, LikeFilter filter) {
        if (filter.getValue() == null) {
                return false;
            }

            String searchString = filter.getValue();

            //percentage is wildcard and underscore singlewildcard
            searchString = searchString.replace("%", ESCAPECHAR + "%");
            searchString = searchString.replace("_", ESCAPECHAR + "_");
            searchString = searchString.replace('*', '%');
            searchString = searchString.replace('?', '_');

            String paramName = getParamName();
            StringBuilder sb = new StringBuilder();

            if(!filter.isCaseSensitive()) {
                sb.append("UPPER(e.").append(filter.getPropertyId()).append(")");
            } else {
                sb.append("e.").append(filter.getPropertyId());
            }

            sb.append(" LIKE ");

            if(!filter.isCaseSensitive()) {
                sb.append("UPPER(:").append(paramName).append(")");
            } else {
                sb.append(":").append(paramName);
            }

            sb.append(" ESCAPE '").append(ESCAPECHAR).append("'");

            String jpql = sb.toString();
            clause.appendStatement(jpql);
            clause.appendParameter(paramName, searchString);
            return true;
    }

}
